%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Basic code of the DFTNET -
%
% This Deep Learning structure was designed for classifying extracted
% patches extracted 'PATCHES' of the diabetic and control group images,
% mentioned in the paper.
%
% IMPORTANT: You need to change in line 17 the address where the DATA
% folder is located
%
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc
clear all

imds = imageDatastore('C:\Users\My Documents\DATA', ...
    'IncludeSubfolders',true, 'FileExtensions','.png', ...
    'LabelSource','foldernames');

% Data without augmentation for validation
[imdsTrainingSet, imdsValidationSet] = splitEachLabel(imds, 0.7, 'randomize');

% Create augmented image data store
% Specify data augmentation options and values/ranges
imageAugmenter = imageDataAugmenter( ...
    'RandRotation',[-90,90], ...
    'RandXTranslation',[-20 20], ...
    'RandYTranslation',[-20 20],...
    'RandXReflection',true,...
    'RandYReflection',true,...
    'RandXScale',[0.5 4],...
    'RandYScale',[0.5 4]);

% Create simple CNN
imageSize = [180 80 3];

% Apply transformations (using randmly picked values) and build augmented
% data store
augImds = augmentedImageDatastore(imageSize,imds,'DataAugmentation',imageAugmenter);

% Create training and validation sets
[augImdsTrainingSet, augImdsValidationSet] = splitEachLabel(imds, 0.7, 'randomize');


% Specify the convolutional neural network architecture.
layers = [
    imageInputLayer((imageSize),'Name','input')
    
    convolution2dLayer(7,32,'Padding','same','Name','conv_1')
    batchNormalizationLayer('Name','BN_1')
    reluLayer('Name','relu_1')
    
    maxPooling2dLayer(3,'Stride',2,'Name','MaxPool_1')
    
    convolution2dLayer(1,64,'Padding','same','Name','conv_2')
    convolution2dLayer(3,64,'Padding','same','Name','conv_2_1')
    batchNormalizationLayer('Name','BN_2')
    reluLayer('Name','relu_2')
    
    maxPooling2dLayer(3,'Stride',2,'Name','MaxPool_2')
    
    convolution2dLayer(3,32,'Padding','same','Name','conv_3')
    batchNormalizationLayer('Name','BN_3')
    reluLayer('Name','relu_3')
    
    maxPooling2dLayer(2,'Stride',2,'Name','MaxPool_3')
    
    convolution2dLayer(3,32,'Padding','same','Name','conv_4')
    batchNormalizationLayer('Name','BN_4')
    reluLayer('Name','relu_4')
    
    fullyConnectedLayer(2,'Name','Fully_Conn')
    softmaxLayer('Name','softmax')
    classificationLayer('Name','classOutput')];


% Specify training options (stochastic gradient descent with momentum)
options = trainingOptions('adam', ...
    'MaxEpochs',100, ...
    'Shuffle','every-epoch', ...    % just study more this part
    'ValidationData',imdsValidationSet, ...
    'ValidationFrequency',15, ...
    'Verbose',false, ...
    'MiniBatchSize',64,...
    'Plots','training-progress');

% Train the network.
net1 = trainNetwork(augImdsTrainingSet,layers,options);

% Report accuracy of baseline classifier on validation set
YPred = classify(net1,imdsValidationSet);
YValidation = imdsValidationSet.Labels;

YPred = grp2idx(YPred);
YPred = YPred';
YValidation = grp2idx(YValidation);
YValidation = YValidation';